@@ -5,6 +5,11 @@ import httpx
 from httpx import Response
 from . import GQLException
 
+
+class TokenExpiredException(Exception):
+    pass
+
+
 TIMEOUT = float(os.getenv("GQL_CLIENT_TIMEOUT", "25"))
 print(f"{TIMEOUT=}")
 
@@ -14,13 +19,20 @@ client = httpx.AsyncClient(timeout=TIMEOUT)
 
 def check_for_errors(j: dict) -> None:
     if errors := j.get("errors"):
+        if "Token is expired" in str(errors):
+            raise TokenExpiredException(errors)
         raise GQLException(errors)
 
 
 def finish(
     *, response: Response, query_str: str, should_print: bool, start_time: float
 ) -> dict:
-    j = response.json()
+    try:
+        j = response.json()
+    except Exception as e:
+        message = f".json() did not work for {response=}"
+        print(message, "-", f"{e=}")
+        raise GQLException(message)
     try:
         print(
             f"took: {(time.time() - start_time) * 1000}, "
@@ -36,23 +48,28 @@ def finish(
     return j
 
 
+"""
+DEP
 async def gql(
     url: str,
     query_str: str,
     variables: dict = None,
+    headers: dict = None,
     should_print: bool = False,
     use_one_time: bool = False,
 ) -> dict:
     start = time.time()
+    headers = headers or {}
     json = {"query": query_str, "variables": variables or {}}
     if use_one_time:
         async with httpx.AsyncClient(timeout=TIMEOUT) as onetime_client:
-            response = await onetime_client.post(url=url, json=json)
+            response = await onetime_client.post(url=url, json=json, headers=headers)
     else:
-        response = await client.post(url=url, json=json)
+        response = await client.post(url=url, json=json, headers=headers)
     return finish(
         response=response,
         query_str=query_str,
         should_print=should_print,
         start_time=start,
     )
+"""
