@@ -8,7 +8,6 @@ from black import format_str, FileMode
 import graphql
 from pydantic import BaseModel
 
-
 NODE_CONFIG_NAME = "GQL"
 ORM_MODEL_NAME = "GQLInput"
 NODE_MODEL_NAME = "Node"
@@ -20,8 +19,11 @@ def imports() -> str:
         "from __future__ import annotations",
         "from enum import Enum",
         "from datetime import datetime, date",
+        "import asyncio",
+        "from pathlib import Path",
         "from pydantic import BaseModel, Field",
         f"from dgraph_orm import {ORM_MODEL_NAME}, Node, GQLException",
+        f"from dgraph_orm.ACL.client import DgraphClient",
         "from dgraph_orm.resolver import Params, Resolver",
         "from typing import Optional, Set, Type, ClassVar, List",
     ]
@@ -131,6 +133,8 @@ class {NODE_CONFIG_NAME}:
 
     url: str = "{self.url}"
     uid_field_name: str = "{self.uid_field_name}"
+    
+    dgraph_client: DgraphClient = client
         """
 
         return s
@@ -461,10 +465,13 @@ async def {edge_name}(self, resolver: {resolver_name} = None, refresh: bool = Fa
         upsert_str = ", upsert: bool = False"
         if not has_upsert:
             upsert_str = ""
+        add_upsert_str = f", upsert=upsert"
+        if not has_upsert:
+            add_upsert_str = ""
         return f"""
 @classmethod
 async def add(cls, *, input: {self.node_config.add_model}, resolver: {self.resolver_name} = None{upsert_str}, use_one_time: bool = False) -> {self.node_name}:
-    return (await cls.add_many(inputs=[input], resolver=resolver, upsert=upsert, use_one_time=use_one_time))[0]
+    return (await cls.add_many(inputs=[input], resolver=resolver{add_upsert_str}, use_one_time=use_one_time))[0]
 
 @classmethod
 async def add_many(cls, *, inputs: List[{self.node_config.add_model}], resolver: {self.resolver_name} = None{upsert_str}, use_one_time: bool = False) -> List[{self.node_name}]:
@@ -763,6 +770,7 @@ class EnhancedJSONEncoder(json.JSONEncoder):
 
 
 def build_all(
+    dgraph_client_str: str,
     schema: graphql.GraphQLSchema,
     node_configs: T.List[NodeConfig],
     include_strawberry: bool = False,
@@ -800,6 +808,7 @@ def build_all(
 
     lines = [
         imports(),
+        dgraph_client_str,
         enums,
         inputs,
         non_input_non_nodes,
@@ -821,9 +830,24 @@ def build_all(
 from gql import gql, Client
 from gql.transport.aiohttp import AIOHTTPTransport
 
+from dgraph_orm.ACL.client import DgraphClient
 
-def schema_from_url(url: str) -> graphql.GraphQLSchema:
-    transport = AIOHTTPTransport(url=url)
+
+async def schema_from_url(
+    url: str,
+    api_key: str = None,
+    user_id: str = None,
+    password: str = None,
+    namespace: int = None,
+) -> graphql.GraphQLSchema:
+    headers = {}
+    if api_key:
+        client = DgraphClient(client_url=url, api_key=api_key)
+        await client.login_into_namespace(
+            user_id=user_id, password=password, namespace=namespace
+        )
+        headers = client.headers
+    transport = AIOHTTPTransport(url=url, headers=headers)
     client = Client(transport=transport, fetch_schema_from_transport=True)
     query = gql(
         """
@@ -836,7 +860,7 @@ def schema_from_url(url: str) -> graphql.GraphQLSchema:
         }
     """
     )
-    client.execute(query)
+    await client.execute_async(query)
     return client.schema
 
 
@@ -860,26 +884,63 @@ def filename_safe(s: str) -> str:
     return "".join([c for c in s if c.isalpha() or c.isdigit() or c == " "]).rstrip()
 
 
-def build_from_app_config(
+def build_dgraph_client(app_config: dict) -> str:
+    client_url, api_key = app_config["url"], app_config.get("api_key")
+    first_line = f'client = DgraphClient.from_cloud(client_url="{client_url}", api_key="{api_key}")'
+    if "namespace" not in app_config:
+        return first_line
+    s = f"""
+{first_line}
+async def login_client() -> None:
+    await client.login_into_namespace(
+        user_id="{app_config['user_id']}",
+        password="{app_config['password']}",
+        namespace={app_config['namespace']},
+    )
+try:
+    loop = asyncio.get_running_loop()
+except RuntimeError:
+    loop = None
+
+if loop and loop.is_running():
+    tsk = loop.create_task(login_client())
+    tsk.add_done_callback(lambda t: print(f"Done logging in for {{Path(__file__).name}}"))
+else:
+    asyncio.run(login_client())
+    """
+    return format_str(s, mode=FileMode())
+
+
+async def build_from_app_config(
     app_name: str, app_config: dict, parent_dir: Path, include_strawberry: bool = False
 ) -> None:
     node_configs = node_configs_from_app_config(
         app_name=app_name, app_config=app_config
     )
-    schema = schema_from_url(app_config["url"])
+    schema = await schema_from_url(
+        app_config["url"],
+        app_config.get("api_key"),
+        app_config.get("user_id"),
+        app_config.get("password"),
+        app_config.get("namespace"),
+    )
+    dgraph_client_str = build_dgraph_client(app_config)
     s = build_all(
-        schema=schema, node_configs=node_configs, include_strawberry=include_strawberry
+        dgraph_client_str=dgraph_client_str,
+        schema=schema,
+        node_configs=node_configs,
+        include_strawberry=include_strawberry,
     )
     open(parent_dir / f"{filename_safe(app_name)}.py", "w").write(s)
 
 
-def build_from_config(
+async def build_from_config(
     config_path: Path, path_to_write_to: Path, include_strawberry: bool = False
 ) -> None:
     path_to_write_to.mkdir(parents=True, exist_ok=True)
     config = json.loads(open(config_path, "r").read())
     for app_name, app_config in config.items():
-        build_from_app_config(
+        await build_from_app_config(
             app_name=app_name,
             app_config=app_config,
             parent_dir=path_to_write_to,
